package com.glxp.api.admin.controller.basic;

import com.github.pagehelper.PageInfo;
import com.glxp.api.admin.res.basic.ErpUnitsResponse;
import com.glxp.api.admin.entity.basic.BasicUnitMaintainEntity;
import com.glxp.api.admin.entity.info.SystemParamConfigEntity;
import com.glxp.api.admin.httpclient.ErpBasicClient;
import com.glxp.api.admin.req.basic.BasicUnitMaintainFilterRequest;
import com.glxp.api.admin.req.basic.BasicUnitMaintainSaveRequest;
import com.glxp.api.admin.req.info.SystemParamConfigRequest;
import com.glxp.api.admin.req.inout.DeleteRequest;
import com.glxp.api.admin.req.receipt.CombineUnitRequest;
import com.glxp.api.admin.req.receipt.UnitMaintainFilterRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.res.basic.BasicUnitMaintainResponse;
import com.glxp.api.admin.service.basic.BasicUnitMaintainService;
import com.glxp.api.admin.service.info.SystemParamConfigService;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 往来单位信息维护
 */
@RestController
public class BasicUnitMaintainController {

    @Resource
    private BasicUnitMaintainService basicUnitMaintainService;

    @Autowired
    private SystemParamConfigService systemParamConfigService;


    @GetMapping("/udiwms/basic/unit/maintain/filter")
    public BaseResponse filterBasicUnitMaintain(BasicUnitMaintainFilterRequest basicUnitMaintainFilterRequest,
                                                BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<BasicUnitMaintainEntity> basicUnitMaintainEntities;
        basicUnitMaintainEntities = basicUnitMaintainService.filterList(basicUnitMaintainFilterRequest);
        PageInfo<BasicUnitMaintainEntity> pageInfo = new PageInfo<>(basicUnitMaintainEntities);
        PageSimpleResponse<BasicUnitMaintainEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(basicUnitMaintainEntities);

        SystemParamConfigEntity systemParamConfigEntityBUMErpId = systemParamConfigService.selectByParamKey("basic_unit_maintain_erp_id");
        SystemParamConfigEntity systemParamConfigEntityBUMThirdId = systemParamConfigService.selectByParamKey("basic_unit_maintain_third_id");
        Map<String, Object> rMap = new HashMap<>();
        rMap.put("BUMErpIdValue",
                systemParamConfigEntityBUMErpId != null && StringUtils.isNotBlank(systemParamConfigEntityBUMErpId.getParamValue()) ?
                        systemParamConfigEntityBUMErpId.getParamValue() : "0");
        rMap.put("BUMThirdIdValue",
                systemParamConfigEntityBUMThirdId != null && StringUtils.isNotBlank(systemParamConfigEntityBUMThirdId.getParamValue()) ?
                        systemParamConfigEntityBUMThirdId.getParamValue() : "0");
        rMap.put("page", pageSimpleResponse);
        return ResultVOUtils.success(rMap);
    }

    @PostMapping("/udiwms/basic/unit/maintain/save")
    public BaseResponse save(@RequestBody @Valid BasicUnitMaintainSaveRequest basicUnitMaintainSaveRequest,
                             BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        BasicUnitMaintainEntity basicUnitMaintainEntity = new BasicUnitMaintainEntity();
        BeanUtils.copyProperties(basicUnitMaintainSaveRequest, basicUnitMaintainEntity);

        boolean b = basicUnitMaintainService.insert(basicUnitMaintainEntity);
        if (!b) {
            return ResultVOUtils.error(ResultEnum.NOT_NETWORK);
        }

        return ResultVOUtils.success();
    }

    @PostMapping("/udiwms/basic/unit/maintain/update")
    public BaseResponse update(@RequestBody BasicUnitMaintainSaveRequest basicUnitMaintainSaveRequest) {
        basicUnitMaintainService.updateById(basicUnitMaintainSaveRequest);
        return ResultVOUtils.success("修改成功");
    }

    @GetMapping("/udiwms/unit/download")
    public BaseResponse downloadBasicUnit(BasicUnitMaintainFilterRequest basicUnitMaintainFilterRequest,
                                          BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<BasicUnitMaintainEntity> basicUnitMaintainEntities;
        basicUnitMaintainEntities = basicUnitMaintainService.filterList(basicUnitMaintainFilterRequest);

        List<BasicUnitMaintainResponse> basicUnitMaintainResponses = basicUnitMaintainEntities.stream().map(item ->
        {
            BasicUnitMaintainResponse basicUnitMaintainResponse = new BasicUnitMaintainResponse();
            BeanUtils.copyProperties(item, basicUnitMaintainResponse);
            return basicUnitMaintainResponse;
        }).collect(Collectors.toList());
        PageInfo<BasicUnitMaintainResponse> pageInfo;
        pageInfo = new PageInfo<>(basicUnitMaintainResponses);
        PageSimpleResponse<BasicUnitMaintainResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(basicUnitMaintainResponses);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @PostMapping("/udiwms/basic/unit/maintain/combine")
    public BaseResponse combine(@RequestBody CombineUnitRequest combineRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<ErpUnitsResponse> custmandocResponses = combineRequest.getCustmandocResponses();
        if (custmandocResponses != null && custmandocResponses.size() > 0) {
            for (ErpUnitsResponse custmandocResponse : custmandocResponses) {
                BasicUnitMaintainEntity basicUnitMaintainEntity = new BasicUnitMaintainEntity();
                basicUnitMaintainEntity.setErpId(custmandocResponse.getId());
                basicUnitMaintainEntity.setName(custmandocResponse.getName());
                basicUnitMaintainEntity.setSpell(custmandocResponse.getSpell());
                basicUnitMaintainEntity.setAddr(custmandocResponse.getAddr());
                basicUnitMaintainEntity.setCreditNo(custmandocResponse.getCreditNo());
                basicUnitMaintainEntity.setContact(custmandocResponse.getContact());
                basicUnitMaintainEntity.setMobile(custmandocResponse.getMobile());
                basicUnitMaintainService.insertBasicUnitMaintain(basicUnitMaintainEntity);
            }
        }

        return ResultVOUtils.success("选入成功");
    }


    @PostMapping("/udiwms/basic/unit/maintain/delete")
    public BaseResponse deleteBasicUnitMaintain(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String id = deleteRequest.getId();

        basicUnitMaintainService.deleteById(id);

        return ResultVOUtils.success("删除成功");
    }


    //查询ERP往来单位
    @GetMapping("/udiwms/unitMaintain/filterErp")
    public BaseResponse filterErp(UnitMaintainFilterRequest unitMaintainFilterRequest,
                                  BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        SystemParamConfigEntity systemParamConfigEntityBUMErpQuery = systemParamConfigService.selectByParamKey("basic_unit_maintain_erp_query");
        if (systemParamConfigEntityBUMErpQuery == null)
            return ResultVOUtils.error(500, "往来单位接口地址未定义");
        BaseResponse<PageSimpleResponse<ErpUnitsResponse>> udiDlDeviceResponse =
                new ErpBasicClient().getErpCrop(unitMaintainFilterRequest, systemParamConfigEntityBUMErpQuery.getParamValue());
        return udiDlDeviceResponse;
    }
}
