package com.glxp.api.admin.controller.basic;

import com.alibaba.fastjson.JSONObject;
import com.glxp.api.admin.entity.basic.UdiInfoEntity;
import com.glxp.api.admin.entity.basic.UdiRelevanceEntity;
import com.glxp.api.admin.entity.info.SystemParamConfigEntity;
import com.glxp.api.admin.entity.basic.UdiEntity;
import com.glxp.api.admin.httpclient.ErpBasicClient;
import com.glxp.api.admin.req.basic.FilterErpGoodsRequest;
import com.glxp.api.admin.req.basic.FilterUdiInfoRequest;
import com.glxp.api.admin.req.info.SystemParamConfigRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.res.basic.ErpProductsResponse;
import com.glxp.api.admin.service.basic.UdiInfoService;
import com.glxp.api.admin.service.basic.UdiRelevanceService;
import com.glxp.api.admin.service.info.SystemParamConfigService;
import com.glxp.api.admin.util.FilterUdiUtils;
import com.glxp.api.admin.util.HttpClient;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

//基础信息维护
@RestController
public class UdiInfoController {

    @Resource
    UdiRelevanceService udiRelevanceService;
    @Resource
    UdiInfoService udiInfoService;
    @Autowired
    private SystemParamConfigService systemParamConfigService;

    @Value("${UDI_SERVER_URL}")
    private String udiUrl;

    //获取同步库UDI信息
    @GetMapping("udiwms/udiinfo/filterUdi")
    public BaseResponse filterUdi(FilterUdiInfoRequest productInfoFilterRequest) {


        String nameCode = null;
        if (productInfoFilterRequest.getNameCode() != null) {
            UdiEntity udiEntity = FilterUdiUtils.getUdi(productInfoFilterRequest.getNameCode());
            if (udiEntity != null) {
                nameCode = udiEntity.getUdi();
            } else {
                nameCode = productInfoFilterRequest.getNameCode();
            }
        }
        Map<String, Object> paramMap = new HashMap<>(16);
        paramMap.put("page", productInfoFilterRequest.getPage());
        paramMap.put("limit", productInfoFilterRequest.getLimit());
        paramMap.put("ylqxzcrbarmc", productInfoFilterRequest.getYlqxzcrbarmc());
        paramMap.put("cpmctymc", productInfoFilterRequest.getCpmctymc());
        paramMap.put("nameCode", nameCode);
        paramMap.put("ggxh", productInfoFilterRequest.getGgxh());
        paramMap.put("uuid", productInfoFilterRequest.getUuid());

        String response = HttpClient.mipsGet(udiUrl + "/udidl/device/filterUdi", paramMap);
        try {
            BaseResponse<PageSimpleResponse<UdiInfoEntity>> udiDlDeviceResponse = (BaseResponse<PageSimpleResponse<UdiInfoEntity>>) JSONObject.parseObject(response, BaseResponse.class);
            return udiDlDeviceResponse;
        } catch (Exception e) {
            e.printStackTrace();
            return ResultVOUtils.error(500, "连接UDI数据下载服务出错！");
        }
    }

    //获取本地已维护UDI信息
    @GetMapping("udiwms/udiinfo/local/filterByUuid")
    public BaseResponse filterLocalByUuid(FilterUdiInfoRequest productInfoFilterRequest) {

        List<UdiInfoEntity> udiInfoEntities = udiInfoService.filterAllUdiInfo(productInfoFilterRequest);
        return ResultVOUtils.success(udiInfoEntities);
    }

    //获取ERP产品信息
    @GetMapping("udiwms/udiinfo/erp/products")
    public BaseResponse getErpProducts(FilterErpGoodsRequest filterErpGoodsRequest,
                                       BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        SystemParamConfigEntity systemParamConfigEntity = systemParamConfigService.selectByParamKey("erp_products_query");
        if (systemParamConfigEntity == null) return ResultVOUtils.error(500, "ERP产品信息接口地址未定义");
        ErpBasicClient erpBasicHttpClient = new ErpBasicClient();
        BaseResponse<PageSimpleResponse<ErpProductsResponse>> udiDlDeviceResponse = erpBasicHttpClient.getErpProducts(filterErpGoodsRequest, systemParamConfigEntity.getParamValue());
        if (udiDlDeviceResponse.getCode() == 20000) {
            List<ErpProductsResponse> invmandocResponses = udiDlDeviceResponse.getData().getList();
            for (ErpProductsResponse invmandocResponse : invmandocResponses) {
                UdiRelevanceEntity udiRelevanceEntity = udiRelevanceService.selectByThirdId(invmandocResponse.getCode());
                if (udiRelevanceEntity != null) {
                    invmandocResponse.setChecked(true);
                } else {
                    invmandocResponse.setChecked(false);
                }

            }
        }
        return udiDlDeviceResponse;

    }
}