package com.glxp.api.admin.controller.basic;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.github.pagehelper.PageInfo;
import com.glxp.api.admin.entity.basic.UdiInfoEntity;
import com.glxp.api.admin.entity.basic.UdiRelevanceEntity;
import com.glxp.api.admin.entity.info.SystemParamConfigEntity;
import com.glxp.api.admin.entity.inout.WarehouseEntity;
import com.glxp.api.admin.httpclient.UdiDlHttpClient;
import com.glxp.api.admin.req.basic.CombineRequest;
import com.glxp.api.admin.req.basic.FilterUdiInfoRequest;
import com.glxp.api.admin.req.basic.PostSmpUdiInfoRequest;
import com.glxp.api.admin.req.inout.DeleteRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.res.basic.UdiRelevanceResponse;
import com.glxp.api.admin.res.inout.WarehouseResponse;
import com.glxp.api.admin.service.basic.UdiInfoService;
import com.glxp.api.admin.service.basic.UdiRelevanceService;
import com.glxp.api.admin.service.info.SystemParamConfigService;
import com.glxp.api.admin.service.inout.CodesService;
import com.glxp.api.admin.thread.BasicUpdateproductService;
import com.glxp.api.admin.util.DateUtil;
import com.glxp.api.admin.util.HttpClient;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@RestController
public class UdiRelevanceController {

    @Value("${UDI_SERVER_URL}")
    private String udiUrl;
    @Resource
    UdiRelevanceService udiRelevanceService;
    @Resource
    UdiInfoService udiInfoService;
    @Resource
    BasicUpdateproductService updateproductService;
    @Resource
    CodesService warehouseService;
    @Resource
    private SystemParamConfigService systemParamConfigService;

    //客户端器械信息下载
    @GetMapping("/udi/udirel/download")
    public BaseResponse downloadProducts(FilterUdiInfoRequest filterUdiInfoRequest,
                                         BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<UdiRelevanceResponse> udiRelevanceResponses = null;
        udiRelevanceResponses = udiRelevanceService.filterUdiRelevance(filterUdiInfoRequest);

        PageInfo<UdiRelevanceResponse> pageInfo;
        pageInfo = new PageInfo<>(udiRelevanceResponses);
        PageSimpleResponse<UdiRelevanceResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(udiRelevanceResponses);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    //器械信息维护首页
    @GetMapping("/udi/udirel/filterUdi")
    public BaseResponse filterUdiGp(FilterUdiInfoRequest filterUdiInfoRequest,
                                    BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<UdiRelevanceResponse> udiRelevanceResponses = null;
        udiRelevanceResponses = udiRelevanceService.filterUdiGp(filterUdiInfoRequest);
        if (udiRelevanceResponses != null && udiRelevanceResponses.size() > 0) {
            for (UdiRelevanceResponse udiRelevanceResponse : udiRelevanceResponses) {
                if (udiRelevanceResponse.getDiType() == 2 && udiRelevanceResponse.getIsUseDy() != 1) {
                    udiRelevanceResponse.setNameCode(udiRelevanceResponse.getSjcpbm());
                }
            }
        }
        PageInfo<UdiRelevanceResponse> pageInfo;
        pageInfo = new PageInfo<>(udiRelevanceResponses);
        PageSimpleResponse<UdiRelevanceResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(udiRelevanceResponses);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    //更新下载ERP存货产品
    @GetMapping("/udi/udirel/updateErp")
    public BaseResponse updateErp() {
        SystemParamConfigEntity systemParamConfigEntity = systemParamConfigService.selectByParamKey("erp_products_query");
        String productUrl;
        String onHandUrl;
        if (systemParamConfigEntity == null)
            return ResultVOUtils.error(500, "ERP产品信息接口地址未定义");
        else
            productUrl = systemParamConfigEntity.getParamValue();
        systemParamConfigEntity = systemParamConfigService.selectByParamKey("erp_onhand_query");
        if (systemParamConfigEntity == null)
            return ResultVOUtils.error(500, "ERP现存量接口地址未定义");
        else
            onHandUrl = systemParamConfigEntity.getParamValue();
        updateproductService.updateProduct(productUrl, onHandUrl);
        return ResultVOUtils.success("后台已开启下载更新，请稍后刷新查看！");
    }

    //多选删除
    @PostMapping("/udi/udirel/deleteAll")
    public BaseResponse deleteAll(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<String> ids = deleteRequest.getIds();
        udiRelevanceService.deleteByIds(ids);
        return ResultVOUtils.success("删除成功");
    }

    @PostMapping("/udi/udirel/deleteById")
    public BaseResponse deleteById(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String ids = deleteRequest.getId();
        UdiRelevanceEntity udiRelevanceEntity = udiRelevanceService.selectById(ids);
        if (udiRelevanceEntity != null && udiRelevanceEntity.getUuid() != null) {
            List<UdiRelevanceEntity> udiRelevanceEntities = udiRelevanceService.selectByUuid(udiRelevanceEntity.getUuid());
            if (udiRelevanceEntities != null && !udiRelevanceEntities.isEmpty()) {
                for (UdiRelevanceEntity udiRelevanceEntity1 : udiRelevanceEntities) {
                    List<UdiInfoEntity> udiInfoEntities = udiInfoService.findByUuids(udiRelevanceEntity1.getUuid());
                    if (udiInfoEntities != null && udiInfoEntities.size() > 0) {
                        for (int i = 0; i < udiInfoEntities.size(); i++) {
                            List<WarehouseEntity> warehouseEntityList = warehouseService.searchCode(udiInfoEntities.get(i).getNameCode());
                            if (warehouseEntityList != null && warehouseEntityList.size() > 0) {
                                return ResultVOUtils.error(500, "改产品已被" + warehouseEntityList.get(0).getOrderId() + "订单关联不能删除");
                            }
                        }
                    }
                }
                udiInfoService.deleteByUuid(udiRelevanceEntity.getUuid());
            }
        }
        udiRelevanceService.deleteById(ids);
        return ResultVOUtils.success("删除成功");
    }

    @PostMapping("/udi/udirel/update")
    public BaseResponse updateRelevanceEntity(@RequestBody UdiRelevanceEntity udiRelevanceEntity, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        udiRelevanceEntity.setUpdateTime(DateUtil.getDateTime());
        udiRelevanceService.updateUdiRelevance(udiRelevanceEntity);
        return ResultVOUtils.success("更新成功");
    }

    //新增或修改关联
    @PostMapping("/udi/udirel/revleance")
    public BaseResponse revleanceUdi(@RequestBody CombineRequest combineRequest) {

        List<String> keys = combineRequest.getKeys();
        if (keys != null && keys.size() > 0) {
            for (String key : keys) {
                FilterUdiInfoRequest filterUdiInfoRequest = new FilterUdiInfoRequest();
                filterUdiInfoRequest.setUuid(key);
                List<UdiInfoEntity> udiInfoEntities = udiInfoService.filterAllUdiInfo(filterUdiInfoRequest);
                UdiRelevanceEntity udiRelevanceEntity = new UdiRelevanceEntity();
                if (udiInfoEntities == null || udiInfoEntities.isEmpty()) {
                    udiInfoEntities = new UdiDlHttpClient(udiUrl).getUdiByUuid(key);
                    if (udiInfoEntities != null) {
                        udiInfoService.insertUdiInfos(udiInfoEntities);
                    } else {
                        return ResultVOUtils.error(500, "连接UDI数据下载服务出错！");
                    }
                }
                if (combineRequest.getRelId() != null) {
                    udiRelevanceEntity.setId(combineRequest.getRelId());
                    udiRelevanceEntity.setUuid(key);
                    udiRelevanceEntity.setUpdateTime(DateUtil.getDateTime());
                    udiRelevanceService.updateUdiRelevance(udiRelevanceEntity);
                } else {
                    udiRelevanceEntity.setThirdId(combineRequest.getThirdId());
                    udiRelevanceEntity.setThirdName(combineRequest.getErpName());
                    udiRelevanceEntity.setUpdateTime(DateUtil.getDateTime());
                    udiRelevanceEntity.setUuid(key);
                    udiRelevanceService.insertUdiRelevance(udiRelevanceEntity);
                }


            }
        }
        return ResultVOUtils.success("合并成功");
    }

    //更换关联ERP产品
    @PostMapping("/udi/udirel/modifyRel")
    public BaseResponse modifyRel(@RequestBody CombineRequest combineRequest) {

        UdiRelevanceEntity udiRelevanceEntity = new UdiRelevanceEntity();
        udiRelevanceEntity.setId(combineRequest.getRelId());
        udiRelevanceEntity.setThirdId(combineRequest.getThirdId());
        udiRelevanceEntity.setThirdName(combineRequest.getErpName());
        udiRelevanceEntity.setUpdateTime(DateUtil.getDateTime());
        udiRelevanceService.updateUdiRelevance(udiRelevanceEntity);
        return ResultVOUtils.success("更新成功");
    }

    //单独选入ERP产品
    @PostMapping("/udi/udirel/addErp")
    public BaseResponse addErp(@RequestBody CombineRequest combineRequest) {
        SystemParamConfigEntity systemParamConfigEntity = systemParamConfigService.selectByParamKey("erp_products_query");
        if (systemParamConfigEntity == null) return ResultVOUtils.error(500, "ERP产品信息接口地址未定义");
        String url = systemParamConfigEntity.getParamValue();
        updateproductService.addProduct(combineRequest.getThirdId(), combineRequest.getErpName(), url);
        return ResultVOUtils.success("添加成功");
    }

    //上传产品信息至供应商管理平台
    @PostMapping("/udiwms/smp/udirl/upload")
    public BaseResponse uploadSMP(@RequestBody CombineRequest combineRequest) {
        SystemParamConfigEntity systemParamConfigEntity = systemParamConfigService.selectByParamKey("smp_uploadProduct_url");
        String postUrl;
        if (systemParamConfigEntity == null)
            return ResultVOUtils.error(500, "上传供应商平台接口地址未定义");
        else
            postUrl = systemParamConfigEntity.getParamValue();
        List<String> ids = combineRequest.getKeys();
        if (ids != null && ids.size() > 0) {
            List<UdiRelevanceResponse> udiRelevanceResponses = new ArrayList<>();
            List<PostSmpUdiInfoRequest>  postSmpUdiInfoRequests = new ArrayList<>();
            for (int i = 0; i < ids.size(); i++) {
                FilterUdiInfoRequest filterUdiInfoRequest = new FilterUdiInfoRequest();
                filterUdiInfoRequest.setId(ids.get(i));
                List<UdiRelevanceResponse> temps = udiRelevanceService.filterUdiRelevance(filterUdiInfoRequest);
                if (temps != null && temps.size() > 0) {
                    udiRelevanceResponses.addAll(temps);
                }
            }
            List<PostSmpUdiInfoRequest> postDatas = udiRelevanceResponses.stream().map(item ->
            {
                PostSmpUdiInfoRequest postSmpUdiInfoRequest = new PostSmpUdiInfoRequest();
                BeanUtils.copyProperties(item, postSmpUdiInfoRequest);
                postSmpUdiInfoRequest.setErpId(item.getThirdId());
                postSmpUdiInfoRequest.setErpName(item.getThirdName());
                return postSmpUdiInfoRequest;
            }).collect(Collectors.toList());
            String response = HttpClient.uCloudPost(postUrl, postDatas);
            BaseResponse baseResponse =
                    JSONObject.parseObject(response, new TypeReference<BaseResponse>() {
                    });
            if (baseResponse.getCode() == 20000) {
                return ResultVOUtils.success("上传成功！");
            } else {
                return ResultVOUtils.error(baseResponse.getCode(), baseResponse.getMessage());
            }
        }else {
            return ResultVOUtils.error(500,"未选择产品信息");
        }



    }


}
