package com.glxp.api.admin.controller.inventory;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.glxp.api.admin.entity.basic.UdiRelevanceEntity;
import com.glxp.api.admin.entity.basic.UdiEntity;
import com.glxp.api.admin.entity.info.SystemParamConfigEntity;
import com.glxp.api.admin.httpclient.ErpInvClient;
import com.glxp.api.admin.req.basic.FilterUdiInfoRequest;
import com.glxp.api.admin.req.basic.UOnhandQueryRequest;
import com.glxp.api.admin.req.inventory.ErpOnhandRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.res.basic.UOnhandQueryResponse;
import com.glxp.api.admin.res.basic.UdiRelevanceResponse;
import com.glxp.api.admin.res.inventory.ErpOnHandResponse;
import com.glxp.api.admin.service.basic.UdiRelevanceService;
import com.glxp.api.admin.service.info.SystemParamConfigService;
import com.glxp.api.admin.util.FilterUdiUtils;
import com.glxp.api.admin.util.HttpClient;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
public class InvErpOnhandController {

    @Resource
    UdiRelevanceService udiRelevanceService;
    @Resource
    private SystemParamConfigService systemParamConfigService;


    @GetMapping("/udiwms/inv/getOnHand")
    public BaseResponse getOnhand(ErpOnhandRequest onhandQueryRequest,
                                  BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (onhandQueryRequest.getUdiCode() != null) {
            UdiEntity udiEntity = FilterUdiUtils.getUdi(onhandQueryRequest.getUdiCode());
            if (udiEntity != null) {
                UdiRelevanceResponse udiRelevanceResponse = findByNamecode(udiEntity.getUdi());
                if (udiRelevanceResponse != null) {
                    String thirdId = udiRelevanceResponse.getThirdId();
                    onhandQueryRequest.setNameCode(thirdId);
                    onhandQueryRequest.setBatchNo(udiEntity.getBatchNo());

                } else {
                    return ResultVOUtils.error(500, "产品信息未维护！");
                }

            } else {
                return ResultVOUtils.error(500, "无效UDI码");

            }
        } else if (onhandQueryRequest.getNameCode() != null) {
            UdiRelevanceResponse udiRelevanceResponse = findByNamecode(onhandQueryRequest.getNameCode());
            onhandQueryRequest.setInventoryCode(udiRelevanceResponse.getThirdId());
        }
        SystemParamConfigEntity systemParamConfigEntity = systemParamConfigService.selectByParamKey("erp_onhand_query");
        String onHandUrl;
        if (systemParamConfigEntity == null)
            return ResultVOUtils.error(500, "ERP现存量接口地址未定义");
        else
            onHandUrl = systemParamConfigEntity.getParamValue();
        BaseResponse<PageSimpleResponse<ErpOnHandResponse>> responseBaseResponse = new ErpInvClient().getOnHandResponse(onhandQueryRequest, onHandUrl);
        return responseBaseResponse;
    }


    public UdiRelevanceResponse findByNamecode(String udi) {
        FilterUdiInfoRequest filterUdiInfoRequest = new FilterUdiInfoRequest();
        filterUdiInfoRequest.setNameCode(udi);
        List<UdiRelevanceResponse> udiRelevanceResponses = udiRelevanceService.filterUdiGp(filterUdiInfoRequest);
        if (udiRelevanceResponses != null && udiRelevanceResponses.size() > 0) {
            return udiRelevanceResponses.get(0);
        } else {
            return null;
        }
    }


}
