package com.glxp.api.admin.controller.inventory;

import com.github.pagehelper.PageInfo;
import com.glxp.api.admin.entity.info.SystemParamConfigEntity;
import com.glxp.api.admin.httpclient.ErpInvClient;
import com.glxp.api.admin.req.inventory.ErpOnhandRequest;
import com.glxp.api.admin.entity.inventory.StockPrintEntity;
import com.glxp.api.admin.req.basic.FilterStockprintRequest;
import com.glxp.api.admin.req.basic.UpdatePrintStatusRequest;
import com.glxp.api.admin.req.inout.DeleteRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.res.inventory.ErpOnHandResponse;
import com.glxp.api.admin.service.inventory.InvStockPrintService;
import com.glxp.api.admin.service.info.SystemParamConfigService;
import com.glxp.api.admin.thread.InvImportStockService;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

@RestController
public class InvPrintController {
    @Resource
    InvStockPrintService stockPrintService;
    @Resource
    InvImportStockService importStockService;
    @Resource
    private SystemParamConfigService systemParamConfigService;

    @PostMapping("/udiwarehouse/stockPrint/filter")
    public BaseResponse filterAll(@RequestBody FilterStockprintRequest stockprintRequest,
                                  BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<StockPrintEntity> stockPrintEntities
                = stockPrintService.filterStockPrint(stockprintRequest);
        PageInfo<StockPrintEntity> pageInfo;
        pageInfo = new PageInfo<>(stockPrintEntities);
        PageSimpleResponse<StockPrintEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(stockPrintEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("/udiwarehouse/stockPrint/getFilter")
    public BaseResponse getFilter(FilterStockprintRequest stockprintRequest,
                                  BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<StockPrintEntity> stockPrintEntities
                = stockPrintService.filterStockPrint(stockprintRequest);
        PageInfo<StockPrintEntity> pageInfo;
        pageInfo = new PageInfo<>(stockPrintEntities);
        PageSimpleResponse<StockPrintEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(stockPrintEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @GetMapping("/udiwarehouse/stockPrint/filterNum")
    public BaseResponse filterNum(FilterStockprintRequest stockprintRequest,
                                  BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        SystemParamConfigEntity systemParamConfigEntity = systemParamConfigService.selectByParamKey("erp_onhand_query");
        String onHandUrl;
        if (systemParamConfigEntity == null)
            return ResultVOUtils.error(500, "ERP产品信息接口地址未定义");
        else onHandUrl = systemParamConfigEntity.getParamValue();

        List<StockPrintEntity> stockPrintEntities
                = stockPrintService.filterStockPrint(stockprintRequest);
        if (stockPrintEntities != null) {
            for (StockPrintEntity stockPrintEntity : stockPrintEntities) {
                ErpOnhandRequest erpOnhandRequest = new ErpOnhandRequest();
                erpOnhandRequest.setInventoryCode(stockPrintEntity.getThirdId());
                List<ErpOnHandResponse> erpOnHandResponses = new ErpInvClient().getOnHand(erpOnhandRequest, onHandUrl);
                if (erpOnHandResponses != null && erpOnHandResponses.size() > 0) {
                    for (ErpOnHandResponse onhandQueryResponse : erpOnHandResponses) {
                        if (onhandQueryResponse.getBatchNo().equals(stockPrintEntity.getBatchNo()) && onhandQueryResponse.getWarehouseCode().equals(stockPrintEntity.getWarehouseCode())) {
                            stockPrintEntity.setNnum(onhandQueryResponse.getCount() + "");
                            break;
                        }
                    }
                }
            }
        }
        PageInfo<StockPrintEntity> pageInfo;
        pageInfo = new PageInfo<>(stockPrintEntities);
        PageSimpleResponse<StockPrintEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(stockPrintEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("/udiwarehouse/stockPrint/import")
    public BaseResponse importStock() {
        importStockService.updateStock();
        return ResultVOUtils.success("后台已开始自动更新，请稍后刷新查看");
    }

    @PostMapping("/udiwarehouse/stockPrint/update")
    public BaseResponse updatePrintStatus(@RequestBody UpdatePrintStatusRequest updatePrintStatusRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<String> ids = updatePrintStatusRequest.getIds();
        if (ids != null && ids.size() > 0) {
            stockPrintService.updatePrintStatus(ids);
        } else {
            return ResultVOUtils.error(500, "更新数据不能为空");
        }
        return ResultVOUtils.success("更新成功");
    }


    @PostMapping("/udiwarehouse/stockPrint/modify")
    public BaseResponse updatePrintStatus(@RequestBody StockPrintEntity stockPrintEntity, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (stockPrintEntity.getProduceDate() == null) {
            stockPrintEntity.setProduceDate("");
        }

        if (stockPrintEntity.getExpireDate() == null) {
            stockPrintEntity.setExpireDate("");
        }

        if (stockPrintEntity.getBatchNo() == null) {
            stockPrintEntity.setBatchNo("");
        }
        String udiCode = "#" + stockPrintEntity.getNameCode() + "#" + stockPrintEntity.getProduceDate() + "#" + stockPrintEntity.getExpireDate() +
                "#" + stockPrintEntity.getBatchNo() + "#" + "#" + stockPrintEntity.getWarehouseCode();
        stockPrintEntity.setUdiCode(udiCode);
        stockPrintService.updateStockPrint(stockPrintEntity);
        return ResultVOUtils.success("更新成功");
    }

    @PostMapping("/udiwarehouse/stockPrint/delete")
    public BaseResponse deletePrintStatus(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        stockPrintService.deleteById(deleteRequest.getId());
        return ResultVOUtils.success("删除成功");
    }



    @GetMapping("/warehouse/stockPrint/dlErpStock")
    public BaseResponse dlErpStock() {
//        dlErpStockService.dlErpStock();
        return ResultVOUtils.success("后台已自动更新，请稍后刷新查看！");
    }

    @GetMapping("/udiwarehouse/stockPrint/importBycode")
    public BaseResponse importBycode(FilterStockprintRequest stockprintRequest) {
        if (stockprintRequest.getThirdId() == null || stockprintRequest.getThirdId().equals("")) {
            return ResultVOUtils.error(500, "存货编码不能为空");
        }

        SystemParamConfigEntity systemParamConfigEntity = systemParamConfigService.selectByParamKey("erp_inv_product_query");
        String url;
        if (systemParamConfigEntity == null)
            return ResultVOUtils.error(500, "ERP库存产品信息查询接口地址未定义");
        else url = systemParamConfigEntity.getParamValue();

        importStockService.updateByCode(stockprintRequest.getThirdId(),url);
        return ResultVOUtils.success("后台已开始自动更新，请稍后刷新查看");
    }


}
