package com.glxp.api.admin.thread;

import com.glxp.api.admin.constant.ConstantType;
import com.glxp.api.admin.entity.basic.UdiInfoEntity;
import com.glxp.api.admin.entity.basic.UdiRelevanceEntity;
import com.glxp.api.admin.entity.info.SystemParamConfigEntity;
import com.glxp.api.admin.entity.inout.ErpOrderEntity;
import com.glxp.api.admin.entity.inout.FilterErpOrderRequest;
import com.glxp.api.admin.entity.inout.OrderEntity;
import com.glxp.api.admin.entity.inventory.ErpStockEntity;
import com.glxp.api.admin.entity.inventory.StockPrintEntity;
import com.glxp.api.admin.httpclient.ErpInvClient;
import com.glxp.api.admin.req.basic.FilterStockprintRequest;
import com.glxp.api.admin.req.basic.FilterUdiInfoRequest;
import com.glxp.api.admin.req.inout.FilterStockCheckRequest;
import com.glxp.api.admin.req.inout.OrderFilterRequest;
import com.glxp.api.admin.req.inventory.ErpOnhandRequest;
import com.glxp.api.admin.res.basic.UdiRelevanceResponse;
import com.glxp.api.admin.res.inventory.ErpInvProductResponse;
import com.glxp.api.admin.res.inventory.ErpOnHandResponse;
import com.glxp.api.admin.service.basic.UdiInfoService;
import com.glxp.api.admin.service.basic.UdiRelevanceService;
import com.glxp.api.admin.service.info.SystemParamConfigService;
import com.glxp.api.admin.service.inout.OrderDetailService;
import com.glxp.api.admin.service.inout.OrderService;
import com.glxp.api.admin.service.inventory.InvStockPrintService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class InvImportStockService {
    @Resource
    private OrderService orderService;
    @Resource
    InvStockPrintService stockPrintService;
    @Resource
    UdiRelevanceService udiRelevanceService;
    @Resource
    private OrderDetailService myErpOrderService;
    @Resource
    private UdiInfoService productInfoService;
    @Resource
    private SystemParamConfigService systemParamConfigService;
    final Logger logger = LoggerFactory.getLogger(InvImportStockService.class);

    @Async
    public void updateStock() {

        int page = 1;
        int limit = 1000;
        FilterUdiInfoRequest filterUdiInfoRequest = new FilterUdiInfoRequest();
        SystemParamConfigEntity systemParamConfigEntity = systemParamConfigService.selectByParamKey("erp_inv_product_query");
        String url;
        if (systemParamConfigEntity != null) {
            url = systemParamConfigEntity.getParamValue();
            while (true) {
                filterUdiInfoRequest.setPage(page);
                filterUdiInfoRequest.setLimit(limit);
                List<UdiRelevanceResponse> udiRelevanceResponses = udiRelevanceService.filterUdiRelevance(filterUdiInfoRequest);
                if (udiRelevanceResponses != null && udiRelevanceResponses.size() > 0) {
                    List<StockPrintEntity> stockPrintEntities = new ArrayList<>();
                    for (UdiRelevanceResponse udiRelevanceResponse : udiRelevanceResponses) {
                        ErpOnhandRequest erpOnhandRequest = new ErpOnhandRequest();
                        erpOnhandRequest.setInventoryCode(udiRelevanceResponse.getThirdId());
                        List<ErpInvProductResponse> uOnhandQueryResponses = new ErpInvClient().getInvPrdoduct(erpOnhandRequest, url);
                        if (uOnhandQueryResponses != null && uOnhandQueryResponses.size() > 0) {
                            for (ErpInvProductResponse onhandQueryResponse : uOnhandQueryResponses) {
                                StockPrintEntity stockPrintEntity = new StockPrintEntity();
                                stockPrintEntity.setBatchNo(onhandQueryResponse.getBatchNo());
                                stockPrintEntity.setNnum(onhandQueryResponse.getCount());
                                stockPrintEntity.setWarehouseName(onhandQueryResponse.getWarehouseName());
                                stockPrintEntity.setWarehouseCode(onhandQueryResponse.getWarehouseCode());
                                stockPrintEntity.setExpireDate(onhandQueryResponse.getExpirationDate());
                                stockPrintEntity.setProduceDate(onhandQueryResponse.getManufacturingDate());
                                setCommonparams(stockPrintEntity, udiRelevanceResponse, stockPrintEntities);
                            }
                        } else {
                            StockPrintEntity stockPrintEntity = new StockPrintEntity();
                            setCommonparams(stockPrintEntity, udiRelevanceResponse, stockPrintEntities);
                        }


                    }
                    stockPrintService.insertStockPrints(stockPrintEntities);
                    if (udiRelevanceResponses.size() < limit) {
                        break;
                    } else {
                        page++;
                    }
                } else {
                    break;
                }
            }
        } else {
            logger.error("ERP库存产品信息查询接口地址未定义！");
        }

    }

    @Async
    public void updateByCode(String code, String url) {
        UdiRelevanceEntity udiRelevanceEntity = udiRelevanceService.selectByThirdId(code);
        UdiRelevanceResponse udiRelevanceResponse = new UdiRelevanceResponse();
        BeanUtils.copyProperties(udiRelevanceEntity, udiRelevanceResponse);
        List<StockPrintEntity> stockPrintEntities = new ArrayList<>();
        ErpOnhandRequest erpOnhandRequest = new ErpOnhandRequest();
        erpOnhandRequest.setInventoryCode(udiRelevanceResponse.getThirdId());
        List<ErpInvProductResponse> uOnhandQueryResponses = new ErpInvClient().getInvPrdoduct(erpOnhandRequest, url);
        if (uOnhandQueryResponses != null && uOnhandQueryResponses.size() > 0) {

            for (ErpInvProductResponse onhandQueryResponse : uOnhandQueryResponses) {
                StockPrintEntity stockPrintEntity = new StockPrintEntity();
                stockPrintEntity.setBatchNo(onhandQueryResponse.getBatchNo());
                stockPrintEntity.setNnum(onhandQueryResponse.getCount() + "");
                stockPrintEntity.setWarehouseName(onhandQueryResponse.getWarehouseName());
                stockPrintEntity.setWarehouseCode(onhandQueryResponse.getWarehouseCode());
                stockPrintEntity.setExpireDate(onhandQueryResponse.getExpirationDate());
                stockPrintEntity.setProduceDate(onhandQueryResponse.getManufacturingDate());
                setCommonparams(stockPrintEntity, udiRelevanceResponse, stockPrintEntities);
            }

        } else {
            StockPrintEntity stockPrintEntity = new StockPrintEntity();
            setCommonparams(stockPrintEntity, udiRelevanceResponse, stockPrintEntities);
        }

    }

    public void setCommonparams(StockPrintEntity stockPrintEntity, UdiRelevanceResponse udiRelevanceResponse, List<StockPrintEntity> stockPrintEntities) {


        stockPrintEntity.setCpmctymc(udiRelevanceResponse.getCpmctymc());
        stockPrintEntity.setThirdName(udiRelevanceResponse.getThirdName());
        stockPrintEntity.setGgxh(udiRelevanceResponse.getGgxh());
        stockPrintEntity.setNameCode(udiRelevanceResponse.getNameCode());
        stockPrintEntity.setThirdId(udiRelevanceResponse.getThirdId());
        stockPrintEntity.setUdiRlIdFk(udiRelevanceResponse.getId() + "");
        stockPrintEntity.setYlqxzcrbarmc(udiRelevanceResponse.getYlqxzcrbarmc());
        stockPrintEntity.setZczbhhzbapzbh(udiRelevanceResponse.getZczbhhzbapzbh());

        stockPrintEntity.setCodeType("3");
//                        if (udiRelevanceResponse.getTyshxydm() != null && !udiRelevanceResponse.getTyshxydm().equals("")) {
//
//                            if (udiRelevanceResponse.getNameCode().contains("MA")) {
//                                stockPrintEntity.setCodeType("2");
//
//                                udiCode = udiRelevanceResponse.getNameCode();
//                                if (udiRelevanceResponse.getBatchNo() != null && !udiRelevanceResponse.getBatchNo().equals("null"))
//                                    udiCode = udiCode + ".L" + udiRelevanceResponse.getBatchNo();
//                            } else {
//                                stockPrintEntity.setCodeType("1");
//                                udiCode = "01" + udiRelevanceResponse.getNameCode();
//
//                                if (udiRelevanceResponse.getBatchNo() != null && !udiRelevanceResponse.getBatchNo().equals("null"))
//                                    udiCode = udiCode + "10" + udiRelevanceResponse.getBatchNo();
//                            }
//                        } else {
//
//                        }
        stockPrintEntity.setUdiCode(generateUdiCode(stockPrintEntity));
        FilterStockprintRequest filterStockprintRequest = new FilterStockprintRequest();
        filterStockprintRequest.setUdiRlIdFk(stockPrintEntity.getUdiRlIdFk() + "");
        filterStockprintRequest.setBatchNo(stockPrintEntity.getBatchNo());
        filterStockprintRequest.setWarehouseCode(stockPrintEntity.getWarehouseCode());
        StockPrintEntity temp = stockPrintService.findByRlId(filterStockprintRequest);
        if (temp != null) {

            //如果生产日期与失效日期都有则更新
            if (stockPrintEntity.getProduceDate() != null && !stockPrintEntity.getProduceDate().equals("") && stockPrintEntity.getExpireDate() != null && !stockPrintEntity.getExpireDate().equals("")) {
                temp.setProduceDate(stockPrintEntity.getProduceDate());
                temp.setExpireDate(stockPrintEntity.getExpireDate());
            }
            if (temp.getProduceDate() == null) {
                temp.setProduceDate(stockPrintEntity.getProduceDate());
            }
            if (temp.getExpireDate() == null) {
                temp.setExpireDate(stockPrintEntity.getExpireDate());
            }
            temp.setUdiCode(generateUdiCode(temp));
            stockPrintEntity = temp;
        }
        if (udiRelevanceResponse.getTyshxydm() != null && !udiRelevanceResponse.getTyshxydm().equals("")) {
            if (udiRelevanceResponse.getDiType() == 1) {
                stockPrintEntity.setNameCode(udiRelevanceResponse.getNameCode());
                stockPrintEntities.add(stockPrintEntity);
            }
        } else
            stockPrintEntities.add(stockPrintEntity);
    }


    public String generateUdiCode(StockPrintEntity stockPrintEntity) {
        String udiCode = "";
        if (stockPrintEntity.getProduceDate() == null) {
            stockPrintEntity.setProduceDate("");
        }

        if (stockPrintEntity.getExpireDate() == null) {
            stockPrintEntity.setExpireDate("");
        }

        if (stockPrintEntity.getBatchNo() == null) {
            stockPrintEntity.setBatchNo("");
        }
        if (stockPrintEntity.getWarehouseCode() == null) {
            stockPrintEntity.setWarehouseCode("");
        }
        //货位码类型
        udiCode = "#" + stockPrintEntity.getNameCode() + "#" + stockPrintEntity.getProduceDate() + "#" + stockPrintEntity.getExpireDate() +
                "#" + stockPrintEntity.getBatchNo() + "#" + "#" + stockPrintEntity.getWarehouseCode();
        return udiCode;
    }

    @Async
    public void checkStock(FilterStockCheckRequest filterStockCheckRequest) {
        SystemParamConfigEntity systemParamConfigEntity = systemParamConfigService.selectByParamKey("erp_inv_product_query");
        String url;
        if (systemParamConfigEntity != null) {
            url = systemParamConfigEntity.getParamValue();

            List<ErpOrderEntity> erpOrderEntitie = getErpOrders(filterStockCheckRequest.getId() + "");
            if (erpOrderEntitie != null && erpOrderEntitie.size() > 0) {
                for (ErpOrderEntity erpOrderEntity : erpOrderEntitie) {
                    //获取UUID
                    FilterUdiInfoRequest productFilterRequest = new FilterUdiInfoRequest();
                    productFilterRequest.setNameCode(erpOrderEntity.getNameCode());
                    List<UdiInfoEntity> productInfoEntities = productInfoService.filterAllUdiInfo(productFilterRequest);
                    Map<String, List<UdiInfoEntity>> groupBy = productInfoEntities.stream().collect(Collectors.groupingBy(UdiInfoEntity::getUuid));
                    List<String> uuids = new ArrayList<>();
                    for (String uuid : groupBy.keySet()) {
                        uuids.add(uuid);
                    }
                    if (uuids.size() == 1) {
                        List<UdiRelevanceEntity> udiRelevanceEntities = udiRelevanceService.selectByUuid(uuids.get(0));
                        if (udiRelevanceEntities != null && udiRelevanceEntities.size() == 1) {
                            ErpOnhandRequest erpOnhandRequest = new ErpOnhandRequest();
                            erpOnhandRequest.setInventoryCode(udiRelevanceEntities.get(0).getThirdId());
                            List<ErpOnHandResponse> erpOnHandResponses = new ErpInvClient().getOnHand(erpOnhandRequest, url);
                            if (erpOnHandResponses != null && erpOnHandResponses.size() > 0) {
                                for (int i = 0; i < erpOnHandResponses.size(); i++) {
                                    if (erpOnHandResponses.get(i).getBatchNo().equals(erpOrderEntity.getBatchNo())
                                    ) {
                                        if (erpOrderEntity.getWarehouseCode() != null && erpOnHandResponses.get(i).getWarehouseCode().equals(erpOrderEntity.getWarehouseCode()))
                                            erpOrderEntity.setReCount(erpOnHandResponses.get(i).getCount());
                                        else {
                                            erpOrderEntity.setReCount(erpOnHandResponses.get(i).getCount());
                                        }
                                        myErpOrderService.updateErpOrder(erpOrderEntity);
                                        break;
                                    }
                                }

                            }

                        }
                    }

                }
            }
        } else {
            logger.error("ERP库存产品信息查询接口地址未定义！");
        }
    }


    private List<ErpOrderEntity> getErpOrders(String id) {
        List<String> orderIds = getOrderIds(id + "");
        List<ErpOrderEntity> erpOrderEntities = new ArrayList<>();
        if (orderIds.size() > 0) {
            FilterErpOrderRequest filterErpOrderRequest = new FilterErpOrderRequest();
            filterErpOrderRequest.setPage(1);
            filterErpOrderRequest.setLimit(10000);
            filterErpOrderRequest.setOrderIds(orderIds);
            erpOrderEntities = myErpOrderService.findByStockCheck(filterErpOrderRequest);
        }
        return erpOrderEntities;
    }

    public List<String> getOrderIds(String checkId) {
        OrderFilterRequest orderFilterRequest = new OrderFilterRequest();
        orderFilterRequest.setAction(ConstantType.TYPE_STOCK_CHECK);
        orderFilterRequest.setStockCheckFk(checkId + "");
        List<OrderEntity> orderEntities = orderService.findAllOrders(orderFilterRequest);
        List<String> orderIds = new ArrayList<>();
        if (orderEntities != null) {
            for (OrderEntity orderEntity : orderEntities) {
                orderIds.add(orderEntity.getId());
            }
        }
        return orderIds;
    }

    private boolean isRepeat(List<ErpOrderEntity> erpOrderEntities) {
        List<String> stringList = erpOrderEntities.stream().map(ErpOrderEntity::getNameCode)
                .collect(Collectors.toList());
        long count = stringList.stream().distinct().count();
        if (stringList.size() == count) {
            return false;
        } else {
            return true;
        }
    }
}
